###################################################################################
# Externe Methode "captcha.py"

# Python imports
import os
import random
import sys
import time
import whrandom
from cStringIO import StringIO
from string import ascii_uppercase, ascii_lowercase, digits

# Zope imports
from DateTime import DateTime
import OFS

# PIL Imports
from PIL import Image, ImageFont, ImageDraw, ImageFilter, ImageOps

# parameters
chars = ascii_uppercase + ascii_lowercase + digits
_xstep = 5
_ystep = 5
_imageSize = (201,61)
_bgColor = (255,255,255) # White
_gridInk = (200,200,200)
_fontInk = (100,100,100)
_fontSize = 24
# set in cb_start callback
_fontPath = sys.path[0] + "/com/zms/captcha/Gastada.ttf"

def getRandomId():
   """ create unique ID with timestamp + 4 random chars
   """
   ts = str(time.time())
   for i in range(4):
     ts += whrandom.choice(chars)
   return ts

def generateImage(folder_path, char_count = 5, font_size = 24, REQUEST=None):
     """ create random image with grey on grey
     """

     zfolder		= folder_path
     _fontSize	= font_size

     # create random chars for the image
     txt = ''
     for i in range(char_count):
         txt += whrandom.choice(ascii_uppercase)

     # set txt to session
     if REQUEST:
       REQUEST.SESSION['sess_captcha'] = txt

     try:
         # recent PIL version with support for truetype fonts
         font = ImageFont.truetype(_fontPath, _fontSize)
     except IOError:
         font = ImageFont.load_default()
     except AttributeError:
         # old PIL version, fallback to pil fonts
         font = ImageFont.load(_fontPath)

     img = Image.new("RGB", _imageSize, _bgColor)
     draw = ImageDraw.Draw(img)

     xsize, ysize = img.size

     # Do we want the grid start at 0,0 or want some offset?
     x, y = 0,0

     while x <= xsize:
         try:
             # recent PIL version
             draw.line(((x, 0), (x, ysize)), fill=_gridInk)
         except TypeError:
             # old PIL version
             draw.setink(_gridInk)
             draw.line(((x, 0), (x, ysize)))
         x = x + _xstep
     while y <= ysize:
         try:
             draw.line(((0, y), (xsize, y)), fill=_gridInk)
         except TypeError:
             draw.setink(_gridInk)
             draw.line(((0, y), (xsize, y)))
         y = y + _ystep

     try:
         draw.text((10, 4), txt, font=font, fill=_fontInk)
     except TypeError:
         draw.setink(_fontInk)
         draw.text((10, 4), txt, font=font)


     file_name = 'testpic.jpg'
     fmt='JPEG'

     imgData = StringIO()
     img.save(imgData, fmt)
     binarydata = imgData.getvalue()
     size = len(binarydata)
     # add Zope Image to ZODB;  taken from OFS/Image.py
     id = getRandomId()
     title = ''
     if REQUEST is not None:
       title = REQUEST.get('REMOTE_ADDR', '') + ' - ' + REQUEST.get('HTTP_USER_AGENT', '')
     content_type = 'image/jpeg'

     # First, we create the image without data
     #zfolder = getattr(self, _ZOPEFOLDERPATH)
     zfolder._setObject(id, OFS.Image.Image(id, title, '', content_type, ''))

     # Now we "upload" the data.  By doing this in two steps, we
     # can use a database trick to make the upload more efficient.
     zope_img = zfolder._getOb(id)
     zope_img.update_data(binarydata, content_type, size)
     #zfolder.manage_addFile(id=id, title=file_name, file=binarydata)

     # delete old images
     img_ids_to_delete = []
     for i in zfolder.objectValues(['Image']):
         mod_datetime = DateTime(i.bobobase_modification_time())
         if int((DateTime() - mod_datetime) * 3600) > 200:
           # longer than 200 minutes
           img_ids_to_delete.append(i.getId())
     zfolder.manage_delObjects(img_ids_to_delete)

     return zope_img

# / Externe Methode "captcha.py"
###################################################################################